@@ -51,8 +51,6 @@ import java.util.Map;
 import java.util.StringTokenizer;
 import java.util.Map.Entry;
 
-import php.java.bridge.http.IContextFactory;
-
 /**
  * This is the main interface of the PHP/Java Bridge. It
  * contains utility methods which can be used by clients.
@@ -76,7 +74,7 @@ public class JavaBridge implements Runnable {
     /**
      * For internal use only. The classloader. 
      */
-    private JavaBridgeClassLoader cl = null;
+    private SimpleJavaBridgeClassLoader cl = null;
 
     static HashMap sessionHash = new HashMap();
 
@@ -172,12 +170,19 @@ public class JavaBridge implements Runnable {
     private ConstructorCache constructorCache = new ConstructorCache();
     StringCache stringCache = new StringCache(this);
 
-    SessionFactory sessionFactory;
+    /** For internal use only. */
+    private IJavaBridgeFactory sessionFactory;
+    
     /** 
      * For internal use only.
      */
     static final SessionFactory defaultSessionFactory = new SessionFactory();
 
+    public IJavaBridgeFactory getFactory() {
+	if(sessionFactory==null) return sessionFactory = defaultSessionFactory;
+	return sessionFactory;
+    }
+    
     Options options;
     
     /**
@@ -195,13 +200,6 @@ public class JavaBridge implements Runnable {
     public void run() {
         try {
 	    logDebug("START: JavaBridge.run()");
-	    setSessionFactory(defaultSessionFactory);
-	    try {
-		setClassLoader(new JavaBridgeClassLoader((DynamicJavaBridgeClassLoader) Thread.currentThread().getContextClassLoader()));
-	    } catch (ClassCastException ex) {
-	         // should never happen
-		setClassLoader(new JavaBridgeClassLoader(null));
-	    }
 	    request = new Request(this);
           
 	    try {
@@ -237,63 +235,6 @@ public class JavaBridge implements Runnable {
     public static ISocketFactory bind(String sockname) throws IOException  {
         return Standalone.bind(Util.logLevel, sockname);
     }
-    /**
-     * parse java.log_file=@HOST:PORT
-     * @param logFile The log file from the PHP .ini file
-     * @return true, if we can use the log4j logger, false otherwise.
-     */
-    private static boolean setChainsawLogger(String logFile) {
-        try {
-	  return doSetChainsawLogger(logFile);
-	} catch (Exception e) {
-	  e.printStackTrace();
-	  Util.setLogger(new FileLogger());
-	}
-	return true;
-    }
-    private static final class Logger extends ChainsawLogger {
-        private String host;
-	private int port;
-	private Logger(String host, int port) {
-	    super();
-	    this.host=host;
-	    this.port=port;
-        }
-        public static Logger createLogger(String host, int port) throws Exception {
-            Logger logger = new Logger(host, port);
-            logger.init();
-	    return logger;
-        }
-        public void configure(String host, int port) throws Exception {
-            host = this.host!=null ? this.host : host;
-            port = this.port > 0 ? this.port : port;
-            super.configure(host, port);
-        }
-    }
-    /**
-     * parse java.log_file=@HOST:PORT
-     * @param logFile The log file from the PHP .ini file
-     * @return true, if we can use the log4j logger, false otherwise.
-     * @throws Exception
-     */
-    private static boolean doSetChainsawLogger(String logFile) throws Exception {
-	if(logFile!=null && logFile.length()>0 && logFile.charAt(0)=='@') {
-	    logFile=logFile.substring(1, logFile.length());
-	    int idx = logFile.indexOf(':');
-	    int port = -1;
-	    String host = null;
-	    if(idx!=-1) {
-		String p = logFile.substring(idx+1, logFile.length());
-		if(p.length()>0) port = Integer.parseInt(p);
-		host = logFile.substring(0, idx);
-	    } else {
-		if(logFile.length()>0) host = logFile;
-	    }
-	    Util.setLogger(Logger.createLogger(host, port));
-	    return true;
-	}
-	return false;
-    }
     /**
      * Global init. Redirects System.out and System.err to the server
      * log file(s) or to System.err and creates and opens the
@@ -317,7 +258,7 @@ public class JavaBridge implements Runnable {
 		rawLogFile=logFile=s.length>0?"":Util.DEFAULT_LOG_FILE;
 		if(s.length>2) {
 		    rawLogFile=logFile=s[2];
-		    if(setChainsawLogger(logFile))
+		    if(Util.setConfiguredLogger(logFile))
 		        logFile=null; // when log4j is used, System.out and System.err are not redirected
 		    else
 		        Util.setLogger(new FileLogger()); // use specified log file
@@ -339,9 +280,8 @@ public class JavaBridge implements Runnable {
 	    Util.logMessage("default logLevel    : " + Util.logLevel);
 	    Util.logMessage("socket              : " + socket);
 	    Util.logMessage("java.ext.dirs       : " + System.getProperty("java.ext.dirs"));
-	    String base = System.getProperty("php.java.bridge.base", System.getProperty("user.home"));
-	    Util.logMessage("php.java.bridge.base: " + base);
-	    Util.logMessage("extra library dir   : " + base+File.separator+"lib");
+	    Util.logMessage("php.java.bridge.base: " + Util.JAVABRIDGE_BASE);
+	    Util.logMessage("extra library dir   : " + Util.JAVABRIDGE_LIB);
 	    Util.logMessage("thread pool size    : " + Util.THREAD_POOL_MAX_SIZE);
 	} catch (Throwable t) {
 	    throw new RuntimeException(t);
@@ -350,10 +290,10 @@ public class JavaBridge implements Runnable {
     // called by Standalone.init()
     static void init(ISocketFactory socket, int logLevel, String s[]) {	    
 	try {
-	    ThreadPool pool = Util.createThreadPool(Util.EXTENSION_NAME+"ThreadPool");
+	    AppThreadPool pool = Util.createThreadPool(Util.EXTENSION_NAME+"ThreadPool");
 	    try {
 	        String policy = System.getProperty("java.security.policy");
-	        String base = System.getProperty("php.java.bridge.base", System.getProperty("user.home"));
+	        String base = Util.JAVABRIDGE_BASE;
 	        if(policy!=null && base!=null) {
 	            SecurityManager manager = new php.java.bridge.JavaBridgeSecurityManager();
 	            System.setSecurityManager(manager);
@@ -370,13 +310,12 @@ public class JavaBridge implements Runnable {
 		Socket sock = socket.accept();
                 Util.logDebug("Socket connection accepted");
 		JavaBridge bridge = new JavaBridge(sock.getInputStream(), sock.getOutputStream());
-                if(pool!=null) {
+		if(pool!=null) {
                     Util.logDebug("Starting bridge thread from thread pool");
 		    pool.start(bridge); // Uses thread pool
 		} else {
                     Util.logDebug("Starting new bridge thread");
 		    Thread t = new Util.Thread(bridge);
-		    t.setContextClassLoader(DynamicJavaBridgeClassLoader.newInstance(Util.getContextClassLoader()));
 		    t.start();
 		}
 	    }
@@ -393,19 +332,13 @@ public class JavaBridge implements Runnable {
      * Note: Do not write anything to System.out, this
      * stream is connected with a pipe which waits for the channel name.
      * @param s an array of [socketname, level, logFile]
-     * @deprecated Use Standalone.main()
+     *  Use Standalone.main()
      * @see php.java.bridge.Standalone#main(String[])
      */
     public static void main(String s[]) {
         Standalone.main(s);
     }
 
-    //
-    // Helper routines which encapsulate the native methods
-    //
-    void setResult(Response response, Object value, Class type) {
-    	response.setResult(value, type);
-    }
 
     /**
      * Print a stack trace to the log file.
@@ -538,7 +471,7 @@ public class JavaBridge implements Runnable {
 		    // for classes which have no visible constructor, return the class
 		    // useful for classes like java.lang.System and java.util.Calendar.
 		    if(createInstance && logLevel>2) {
-		    	logMessage("No visible constructor found in: "+ name +", returning the class instead of an instance; this may not be what you want. Please correct this error or please use new JavaClass(\""+name+"\") instead.");
+		    	logMessage("No visible constructor found in: "+ name +", returning the class instead of an instance; this may not be what you want. Please correct this error or please use the function java(\""+name+"\") instead.");
 		    }
 		    response.setResultClass(clazz);
 		    return;
@@ -550,9 +483,9 @@ public class JavaBridge implements Runnable {
 	    if (this.logLevel>4) {
 	        Object result = selected.newInstance(coercedArgs);
 	        logInvoke(result, name, coercedArgs);
-    	    	response.setResultObject(result);
+    	    	response.setResult(result, clazz);
 	    } else {
-  	    	response.setResultObject(selected.newInstance(coercedArgs));
+  	    	response.setResult(selected.newInstance(coercedArgs), clazz);
   	    }
 	} catch (Throwable e) {
 	    if(e instanceof InvocationTargetException) e = ((InvocationTargetException)e).getTargetException();
@@ -562,7 +495,7 @@ public class JavaBridge implements Runnable {
 		throw (OutOfMemoryError)e; // abort
 	    }
 	    if(e instanceof NoClassDefFoundError) {
-		getClassLoader().clearCaches();
+		getClassLoader().reset();
 		e = getUnresolvedExternalReferenceException(e, "call constructor");
 	    }
 	    setException(response, e, createInstance?"CreateInstance":"ReferenceClass", null, name, args, params);
@@ -670,7 +603,7 @@ public class JavaBridge implements Runnable {
 		if(arg instanceof PhpProcedureProxy) {
 		    PhpProcedureProxy proxy = ((PhpProcedureProxy)arg);
 		    if(proxy.suppliedInterfaces==null) {
-		        continue; // exact match
+			arg = args[i] = proxy.getNewFromInterface(null);
 		    } else {
 			arg = args[i] = proxy.getProxy(null);        
 		    }
@@ -1119,9 +1052,9 @@ public class JavaBridge implements Runnable {
 	        logInvoke(object, method, coercedArgs); 
 	        Object result = selected.invoke(object, coercedArgs);
 	        logResult(result);
-	        setResult(response, result, selected.getReturnType());
+	        response.setResult(result, selected.getReturnType());
 	    } else {
-	        setResult(response, selected.invoke(object, coercedArgs), selected.getReturnType());	      
+	        response.setResult(selected.invoke(object, coercedArgs), selected.getReturnType());	      
 	    }
 	} catch (Throwable e) {
 	    if(e instanceof InvocationTargetException) e = ((InvocationTargetException)e).getTargetException();
@@ -1131,7 +1064,7 @@ public class JavaBridge implements Runnable {
 		throw (OutOfMemoryError)e; // abort
 	    }
 	    if(e instanceof NoClassDefFoundError) {
-		getClassLoader().clearCaches();
+		getClassLoader().reset();
 		e = getUnresolvedExternalReferenceException(e, "call the method");
 	    }
 	    
@@ -1251,7 +1184,7 @@ public class JavaBridge implements Runnable {
 			    } else {
 			    	res=fld.get(object);
 			    }
-			    setResult(response, res, ctype);
+			    response.setResult(res, ctype);
 			    return;
 			}
 		    }
@@ -1281,7 +1214,7 @@ public class JavaBridge implements Runnable {
 				matches.clear();
 				break again1;
 			    }
-			    setResult(response, method.invoke(object, args), method.getReturnType());
+			    response.setResult(method.invoke(object, args), method.getReturnType());
 			    return;
 			}
 		    }
@@ -1312,7 +1245,7 @@ public class JavaBridge implements Runnable {
 			    } else {
 				res = fld.get(object);
 			    }
-			    setResult(response, res, ctype);
+			    response.setResult(res, ctype);
 			    return;
 			}
 		    }
@@ -1328,7 +1261,7 @@ public class JavaBridge implements Runnable {
 		throw (OutOfMemoryError)e; // abort
 	    }
 	    if(e instanceof NoClassDefFoundError) {
-		getClassLoader().clearCaches();
+		getClassLoader().reset();
 		e = getUnresolvedExternalReferenceException(e, "invoke a property");
 	    }
 	    setException(response, e, set?"SetProperty":"GetProperty", object, prop, args, params);
@@ -1425,21 +1358,30 @@ public class JavaBridge implements Runnable {
      * @param out the OutputStream
      * @see php.java.bridge.JavaBridgeRunner
      * @see php.java.bridge.JavaBridge#main(String[])
-     */
-    public JavaBridge(InputStream in, OutputStream out) {
+    */
+    protected JavaBridge(InputStream in, OutputStream out) {
 	this.in = in;
 	this.out = out;
+	this.setClassLoader(this.getFactory().getJavaBridgeClassLoader());
     }
-
+ 
     /**
      * Only for internal use.
      * @see php.java.bridge.JavaBridge#main(String[])
      * @see php.java.bridge.JavaBridgeRunner
      */
-    public JavaBridge() {
+    protected JavaBridge() {
 	this(null, null);
     }
     
+    /**
+     * Create a new bridge using a factory.
+     * @param factory The session/context factory.
+     */
+    public JavaBridge(IJavaBridgeFactory factory) {
+	setFactory(factory);
+	setClassLoader(factory.getJavaBridgeClassLoader());
+    }
     /**
      * Return map for the value (PHP 5 only)
      * @param value - The value which must be an array or implement Map or Collection.
@@ -1451,11 +1393,16 @@ public class JavaBridge implements Runnable {
     }
 
     /**
-     * @deprecated
-     * @see #updateJarLibraryPath(String, String)
+     * Append the path to the current library path<br>
+     * Examples:<br>
+     * setJarLibPath(";file:///tmp/test.jar;file:///tmp/my.jar");<br>
+     * setJarLibPath("|file:c:/t.jar|http://.../a.jar|jar:file:///tmp/x.jar!/");<br>
+     * @param path A file or url list, usually separated by ';'
+     * @param extensionDir The php extension directory. 
+     * @throws IOException 
      */
-    public void setJarLibraryPath(String path, String extensionDir) throws IOException {
-        updateJarLibraryPath(path, extensionDir);
+    public void updateJarLibraryPath(String path, String extensionDir) throws IOException {
+    	updateJarLibraryPath(path, extensionDir, null, null);
     }
     /**
      * Append the path to the current library path<br>
@@ -1466,24 +1413,26 @@ public class JavaBridge implements Runnable {
      * @param extensionDir The php extension directory. 
      * @throws IOException 
      */
-    public void updateJarLibraryPath(String path, String extensionDir) throws IOException {
-    	getClassLoader().updateJarLibraryPath(path, extensionDir.intern());
-    }
-
-    /**
-     * @deprecated
-     * @see #updateLibraryPath(String, String)
-     */
-    public void setLibraryPath(String rawPath, String extensionDir) {
-        updateLibraryPath(rawPath, extensionDir);
+    public void updateJarLibraryPath(String path, String extensionDir, String cwd, String searchpath) throws IOException {
+    	getClassLoader().updateJarLibraryPath(path, extensionDir.intern(), cwd, searchpath);   	
     }
+    
     /**
      * Update the library path for ECMA dll's
      * @param rawPath A file or url list, usually separated by ';'
      * @param extensionDir The php extension directory. 
      */
     public void updateLibraryPath(String rawPath, String extensionDir) {
-        if(rawPath==null || rawPath.length()<2) return;
+    	updateLibraryPath(rawPath, extensionDir, null, null);
+    }
+    /**
+     * Update the library path for ECMA dll's
+     * @param rawPath A file or url list, usually separated by ';'
+     * @param extensionDir The php extension directory. 
+     */
+    public void updateLibraryPath(String rawPath, String extensionDir, String cwd, String searchpath) {
+    	
+    	if(rawPath==null || rawPath.length()<2) return;
         String contextDir = new File(extensionDir, "lib").getAbsolutePath();
         // add a token separator if first char is alnum
 	char c=rawPath.charAt(0);
@@ -1504,6 +1453,15 @@ public class JavaBridge implements Runnable {
 		} else if ((f=new File(contextDir, s)).isFile()) {
 		    buf.append(f.getAbsolutePath());
 		    Util.loadFileMethod.invoke(Util.CLRAssembly, new Object[] {buf.toString()} );
+		} else if ((f=new File(Util.JAVABRIDGE_LIB, s)).isFile()) {
+		    buf.append(f.getAbsolutePath());
+		    Util.loadFileMethod.invoke(Util.CLRAssembly, new Object[] {buf.toString()} );
+		} else if ((f=JarLibraryPath.checkSearchPath(s, searchpath))!=null) {
+		    buf.append(f.getAbsolutePath());
+		    Util.loadFileMethod.invoke(Util.CLRAssembly, new Object[] {buf.toString()} );
+		} else if ((cwd!=null) && (f=new File(cwd, s)).isFile()) {
+		    buf.append(f.getAbsolutePath());
+		    Util.loadFileMethod.invoke(Util.CLRAssembly, new Object[] {buf.toString()} );
 		} else {
 		    buf.append(s);
 		    Util.loadMethod.invoke(Util.CLRAssembly, new Object[] {buf.toString()} );
@@ -1600,6 +1558,19 @@ public class JavaBridge implements Runnable {
 	    return (String)castToString(ob);
     }
 
+    /**
+     * Returns a string representation of the object
+     * @param ob The object
+     * @return A string representation.
+     */
+    public String ObjectToString(byte[] ob) {
+	   try {
+	        return new String ((byte[])castToString(ob), options.getEncoding());
+	    } catch (UnsupportedEncodingException e) {
+		return new String ((byte[])castToString(ob));
+	    }
+    }
+ 
     /**
      * Returns a string representation of the object
      * @param ob The Throwable
@@ -1636,18 +1607,6 @@ public class JavaBridge implements Runnable {
 	if(contextCache!=null) return contextCache;
     	return contextCache = sessionFactory.getContext();
     }
-    /**
-     * Returns the JSR223 context when using persistent connections.
-     * @return The JSR223 context.
-     * @param id The fresh context id, as specified by the jsr223 client, will be aliased to the current id
-     * @return The JSR223 context.
-     * @see #recycle()
-     * @see #getContext()
-     */
-    public Object getContext(String id) {
-	recycleContext(id);
-    	return getContext();
-    }
     private ISession sessionCache = null;
     /**
      * Return a session handle shared among all JavaBridge
@@ -1671,21 +1630,6 @@ public class JavaBridge implements Runnable {
 	  throw t;
 	}
     }
-    /**
-     * Return a session handle when using persistent connections.
-     * @throws Exception 
-     * @param id The fresh context id, as specified by the jsr223 client, will be aliased to the current id
-     * @param name The session name, if any
-     * @param clientIsNew true, if the client wants a new session
-     * @param timeout session timeout in seconds
-     * @return The session context.
-     * @see #recycle()
-     * @see #getSession(String, boolean, int)
-     */
-    public ISession getSession(String id, String name, boolean clientIsNew, int timeout) throws Exception {
-	recycleContext(id);
-	return getSession(name, clientIsNew, timeout);
-    }
     
     /**
      * Create a dynamic proxy proxy for calling PHP code.<br>
@@ -1785,7 +1729,7 @@ public class JavaBridge implements Runnable {
      * implement session sharing.
      * @param sessionFactory The sessionFactory to set.
      */
-    public void setSessionFactory(SessionFactory sessionFactory) {
+    public void setFactory(IJavaBridgeFactory sessionFactory) {
         this.sessionFactory = sessionFactory;
     }
 
@@ -1825,19 +1769,22 @@ public class JavaBridge implements Runnable {
     	session.put(id, obj);
     	return (String)castToString(id);
     }
+    
+    private SimpleJavaBridgeClassLoader defaultClassLoader=null;
     /**
      * Set a new ClassLoader
      * @param cl The ClassLoader
      * 
      */
-    public void setClassLoader(JavaBridgeClassLoader cl) {
+    public void setClassLoader(SimpleJavaBridgeClassLoader cl) {
+	defaultClassLoader = this.cl;
 	this.cl = cl;
     }
     /**
      * Return the current ClassLoader
      * @return The ClassLoader.
      */
-    public JavaBridgeClassLoader getClassLoader() {
+    public SimpleJavaBridgeClassLoader getClassLoader() {
 	return cl;
     }
     /**
@@ -1965,15 +1912,15 @@ public class JavaBridge implements Runnable {
 	return offsetGet((Map)table, off);	
     }
     /**
-     * Selects the asynchronuous protocol mode.
+     * Selects the asynchronous protocol mode.
      */
     public void beginDocument() {
 	Response res = request.response;
-	res.setAsyncWriter();
+	res.setObjectWriter();
 	lastAsyncException = null;
     }
     /**
-     * Back to synchronuous protocol mode
+     * Back to synchronous protocol mode
      */
     public void endDocument() throws Throwable {
 	Response res = request.response;
@@ -2029,9 +1976,19 @@ public class JavaBridge implements Runnable {
 	this.sessionCache = null;
         globalRef = new GlobalRef();
         lastException = lastAsyncException = null;
-	cl.recycle();
+	
+        /* resets the classLoader: make sure to set the default loader before calling sessionFactory.recycle() */
+        if(defaultClassLoader!=null)  {
+	    cl = defaultClassLoader;
+	    defaultClassLoader = null;
+        }
+        cl.recycle();
+        
         options.recycle();
+        
+        /* resets the bridge: make sure to set the original bridge before calling sessionFactory.recycle() */
         request.recycle();
+        /* resets the currentThreadContextClassLoader from the bridge's loader */
         sessionFactory.recycle();
         
         // TODO: recycle common entries such as bridge.require(), etc.
@@ -2039,35 +1996,6 @@ public class JavaBridge implements Runnable {
         constructorCache.clear();
         stringCache.clear();
     }
-    /**
-     * The php client calls this (through getContext or getSession),
-     * if it is running with a different context id than the one
-     * suggested from its (e.g.: PhpCgiServlet- or FacesServlet-)
-     * client.  Example: The PhpCgiServlet creates a context id and
-     * passes it to the php client. Usually the client passes this id
-     * back to the PhpJavaServlet which re-directs to a private
-     * communication channel. However, when
-     * java.persistent_connections is switched on and the client is
-     * already connected to one of PhpJavaServlet's communication
-     * channels (and it is therefore running on a different context id
-     * than the one suggested by the PhpCgiServlet), the PhpCgiServlet
-     * needs some way to pass the fresh information from the suggested
-     * id to the recycled id. Therefore all clients of php clients can implement
-     * a <code>recycle(ContextFactory target)</code> callback which
-     * allows them to attach the information to the recycled
-     * context. The suggested way to do this is to extend SimpleContextFactory
-     * which already implements a suitable recycle and visit method.
-     * 
-     * @param id The suggested context id (from the client of the php client).
-     * @see php.java.bridge.http.ContextFactory#recycle(ContextFactory)     
-     */    
-    private void recycleContext(String id) {
-      // defaultSessionFactory is shared among *all* clients
-      if(sessionFactory==defaultSessionFactory) return;
-      // sessionFactory != default: must be a ContextFactory
-      IContextFactory current = ((IContextFactory)sessionFactory);
-      current.recycle(id);
-  }
     
     /**
      * Return a new string using the current file encoding (see java_set_file_encoding()).
@@ -2103,4 +2031,12 @@ public class JavaBridge implements Runnable {
 	if(options.passContext()) return new ClassicResponse(this);
 	return new Response(this);
     }
+    
+    public boolean typeExists(String name) {
+	try {
+	    getClassLoader().forName(name);
+	    return true;
+	} catch (ClassNotFoundException ex) {/*ignore*/}
+	return false;
+    }
 }
