@@ -253,6 +253,21 @@ public class Response {
 	    return true;
 	}
   }
+    protected abstract class IncompleteArrayValueWriter extends WriterWithDelegate {
+	      
+	public boolean setResult(Object value) {
+	    if (value == null) {
+		writeNull();
+	    } else if (value instanceof PhpString) {
+	        writeString(((PhpString)value).getBytes());
+	    } else if (value instanceof PhpExactNumber) {
+	        writeLong(((PhpExactNumber)value).longValue());
+	    } else {
+	        return false;
+	    }
+	    return true;
+	}
+  }
 
     protected class ClassicWriter extends IncompleteClassicWriter {
  
@@ -266,7 +281,6 @@ public class Response {
     protected class DefaultWriter extends WriterWithDelegate {
 
 	public boolean setResult(Object value) {
-	    if(value==null) {writeNull(); return true;}
 	    if(staticType.isPrimitive()) {
    		if(staticType == Boolean.TYPE)
 		    writeBoolean(((Boolean) value).booleanValue());
@@ -278,21 +292,25 @@ public class Response {
 		    writeDouble(((Number)value).doubleValue());
    		else if(staticType == Character.TYPE) 
 		    writeString(Util.stringValueOf(value));
-   		else { Util.logFatal("Unknown type"); writeObject(value); }
-	    } else if(value instanceof PhpString) //  No need to check for Request.PhpNumber, this cannot happen.
-	        writeString(((PhpString)value).getBytes());
-	    else if(!delegate.setResult(value))
+   		else if(staticType == java.lang.Void.TYPE)
+   		    writeVoid();
+  		else { Util.logFatal("Unknown type"); writeObject(value); }
+	    } else if(value instanceof PhpString) {//  No need to check for Request.PhpNumber, this cannot happen.
+		if(value==null) writeNull();
+		else writeString(((PhpString)value).getBytes());
+	    } else if(!delegate.setResult(value)) {
 		writeObject(value);
+	    }
 	    return true;
         }        	     	
     }
 
-    protected class ArrayValueWriter extends IncompleteClassicWriter {
+    protected class ArrayValueWriter extends IncompleteArrayValueWriter {
 	public void setResult(Object value, Class type) {
 	    if(!delegate.setResult(value)) setResultArray(value);
 	}
 	public boolean setResult(Object value) {
-	    if(!super.setResult(value)) writeString(Util.stringValueOf(value));
+	    if(!super.setResult(value)) writeObject(value);
 	    return true;
 	}
 	private boolean setResultArray(Object value) {
@@ -304,15 +322,35 @@ public class Response {
 	    return true;
 	}
     }
-  	
-    protected final class AsyncWriter extends Writer {
-	public void setResultProcedure(long object, String cname, String name, Object[] args) {
-	    throw new IllegalStateException("Cannot call "+name+": callbacks not allowed in stream mode");
-	}
+    /** Writer used by the async protocol. It always returns <V ...> or <O ...>, even for NULL, Class or Exception values. 
+     * When the client-side cache is enabled, the client will select an Async or AsyncVoidWriter in the next call. */
+    protected class ObjectWriter extends Writer {
 	public void setResultException(Throwable o, String str) {
 	    setResultObject(bridge.lastAsyncException = o);
 	}
 	public void setResultObject(Object value) {
+	    if(staticType == java.lang.Void.TYPE)  { writeVoid(); return; }
+
+	    if(value==null) value=Request.PHPNULL;
+ 	    writeObject(value);
+	}
+	public void setResultClass(Class value) {
+	    if(value==null) value=Request.PhpNull.class; // shouldn't happen
+	    writeClass(value);
+	}
+	public boolean setResult(Object value) { 
+	    setResultObject(value);
+	    return true;
+	}
+    }
+    /** Writer used by the async protocol. It writes nothing but stores the result in global ref*/
+    protected class AsyncWriter extends Writer {
+	public void setResultException(Throwable o, String str) {
+	    setResultObject(bridge.lastAsyncException = o);
+	}
+	public void setResultObject(Object value) {
+	    if(staticType == java.lang.Void.TYPE)  throw new IllegalStateException ("Use the AsyncVoidWriter instead");
+	    
  	    if(value==null) value=Request.PHPNULL;
  	    if(bridge.logLevel>=4) {
 		writeObject(value);
@@ -332,9 +370,6 @@ public class Response {
 	    setResultObject(value);
 	    return true;
 	}
-	public void reset() {
-	    buf.reset();
-	}
 	/**
 	 * Called at the end of each packed.
 	 */
@@ -345,7 +380,27 @@ public class Response {
      	    reset();
 	}
     }
-  	
+    /** Writer used by the async protocol. It writes nothing and doesn't create a result */
+    protected final class AsyncVoidWriter extends Writer {
+	public void setResultProcedure(long object, String cname, String name, Object[] args) {
+	    throw new IllegalStateException("Cannot call "+name+": callbacks not allowed in stream mode");
+	}
+	public void setResultException(Throwable o, String str) {
+	    bridge.lastAsyncException = o;
+	}
+	public void setResultObject(Object value) {}
+	public void setResultClass(Class value) {}
+	public boolean setResult(Object value) { 
+	    return true;
+	}
+	public void flush() throws IOException {
+     	    if(bridge.logLevel>=4) {
+       	     bridge.logDebug(" |<- <NONE>");
+            }
+     	    reset();
+	}	
+    }
+ 	
     protected class CoerceWriter extends Writer {
 	public void setResult(Object value, Class resultType) {
 	    // ignore resultType and use the coerce type
@@ -376,6 +431,8 @@ public class Response {
 		    }
 		} else if(staticType == Character.TYPE) {
 		    writeString(Util.stringValueOf(value));
+		} else if(staticType == java.lang.Void.TYPE) {
+   		    writeVoid();
 		} else { Util.logFatal("Unknown type"); writeObject(value); }
 	    } else if(staticType == String.class) {
 		 if (value instanceof byte[])
@@ -406,7 +463,8 @@ public class Response {
     }
      
     private WriterWithDelegate defaultWriter;
-    private Writer writer, currentWriter, arrayValuesWriter=null, arrayValueWriter=null, coerceWriter=null, asyncWriter=null;
+    private Writer writer, currentWriter, arrayValuesWriter=null, arrayValueWriter=null, coerceWriter=null; 
+    private Writer asyncWriter=null, asyncVoidWriter=null, objectWriter = null;
 
     protected HexOutputBuffer createBase64OutputBuffer() {
 	return new Base64OutputBuffer(bridge);
@@ -456,13 +514,7 @@ public class Response {
     public void setResultException(Throwable value, String asString) {
      	writer.setResultException(value, asString);
     }
-    /**
-      * Set the result packet.
-      * @param value The result object.
-      */
-    public void setResultObject(Object value) {
-     	writer.setResultObject(value);
-    }
+
     /**
       * Set the result packet.
       * @param value The result object.
@@ -484,21 +536,6 @@ public class Response {
 	setDefaultWriter();
         writer.setFinish(keepAlive);
     }
-    /**
-     * Selects a different writer.
-     * @param writerType Must be Response#VALUES_WRITER or Response#COERCE_WRITER.
-     * @return The seleted writer.
-     * @see Response#VALUES_WRITER
-     * @see Response#COERCE_WRITER
-     * @deprecated Use setArrayValuesWriter or setCoerceWriter instead.
-     */
-    public Writer selectWriter(int writerType) {
-     	switch(writerType) {
-    	case VALUES_WRITER: return setArrayValuesWriter();
-    	case COERCE_WRITER: return setCoerceWriter();
-    	default: throw new IllegalArgumentException(String.valueOf(writerType));
-    	}
-    }
     /**
      * Selects a specialized writer which writes objects as an array.
      * Used by castToArray().
@@ -529,7 +566,23 @@ public class Response {
      * @return The async. writer
      */
     public Writer setAsyncWriter() {
-	return currentWriter = getAsyncWriter();
+	return writer = getAsyncWriter();
+    }
+    /**
+     * Selects a specialized writer which does not write anything and does not generate a result proxy
+     * Used by async. protocol.
+     * @return The async. writer
+     */
+    public Writer setAsyncVoidWriter() {
+	return writer = getAsyncVoidWriter();
+    }
+    /**
+     * Selects a specialized writer which always writes <O ...> or <V ..>, even for NULL, Class and Exception values.
+     * This triggers the client cache so that it selects the AsyncNull or AsyncWriter.
+     * @return The async. writer
+     */
+    public Writer setObjectWriter() {
+	return currentWriter = getObjectWriter();
     }
     /**
      * Selects the default writer
@@ -550,7 +603,9 @@ public class Response {
     static final byte[] E="<E v=\"".getBytes();
     static final byte[] O="<O v=\"".getBytes();
     static final byte[] N="<N i=\"".getBytes();
+    static final byte[] V="<V i=\"".getBytes();
     static final byte[] Nul="<N />".getBytes();
+    static final byte[] Void="<V />".getBytes();
     static final byte[] m="\" m=\"".getBytes();
     static final byte[] n="\" n=\"".getBytes();
     static final byte[] p="\" p=\"".getBytes();
@@ -591,6 +646,9 @@ public class Response {
 	buf.append(D); buf.append(d);
 	buf.append(e);
     }
+    void writeVoid() {
+	buf.append(Void);
+    }
     void writeNull() {
 	buf.append(Nul);
     }
@@ -606,16 +664,20 @@ public class Response {
         }
         return po;
     }
+    /** write object (class, interface, object) */
     void writeObject(Object o) {
         if(o==null) { writeNull(); return; }
         Class dynamicType = o.getClass();
 	buf.append(O); buf.append(this.bridge.globalRef.append(o));
+	buf.append(m); buf.append(Util.toBytes(dynamicType.getName()));
 	buf.append(getType(dynamicType));
 	buf.append(e);
     }
+    /** write object of type class. Only called from new JavaClass(...) */
     void writeClass(Class o) {
         if(o==null) { writeNull(); return; }
     	buf.append(O); buf.append(this.bridge.globalRef.append(o));
+	buf.append(m); buf.append(Util.toBytes(o.getName()));
     	buf.append(po);
     	buf.append(e);
     }
@@ -718,8 +780,16 @@ public class Response {
         if(coerceWriter==null) return coerceWriter = new CoerceWriter();
         return coerceWriter;
     }
+    private Writer getObjectWriter() {
+	if(objectWriter==null) return objectWriter = new ObjectWriter();
+	return objectWriter;
+    }
     private Writer getAsyncWriter() {
-      if(asyncWriter==null) return asyncWriter = new AsyncWriter();
-      return asyncWriter;
+	if(asyncWriter==null) return asyncWriter = new AsyncWriter();
+	return asyncWriter;
+    }
+    private Writer getAsyncVoidWriter() {
+	if(asyncVoidWriter==null) return asyncVoidWriter = new AsyncVoidWriter();
+	return asyncVoidWriter;
     }
 }
