@@ -34,12 +34,17 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
+import java.text.DateFormat;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.LinkedList;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
+import java.util.Set;
 import java.util.StringTokenizer;
+import java.util.TimeZone;
 import java.util.Vector;
 
 /**
@@ -76,11 +81,12 @@ public final class Util {
             logger = new FileLogger(); // log to logStream        
         }
         /**
-         * Use chainsaw, if available of the specified logger.
+         * Use chainsaw, if available.
          * @param logger The specified logger.
          */
         public Logger(ILogger logger) {
-            try {this.clogger = ChainsawLogger.createChainsawLogger();} catch (Throwable t) {
+            try {this.clogger = Log4jLogger.createChainsawLogger();} catch (Throwable t) {
+        	if(Util.logLevel>5) t.printStackTrace();
                 this.logger = logger;
             }
         }
@@ -121,6 +127,11 @@ public final class Util {
 		}
 	}
     }
+    
+    /** The default HTTP port for management clients */
+    public static final int HTTP_PORT_BASE = 8080;
+    /** The default Port for Mono and .NET clients */
+    public static int EXTENSION_TCP_PORT_BASE;
 
     /**
      * The default CGI locations: <code>"/usr/bin/php-cgi"</code>, <code>"c:/php/php-cgi.exe</code>
@@ -162,6 +173,10 @@ public final class Util {
      */
     public static final String DEFAULT_EXT_DIRS[] = { "/usr/share/java/ext", "/usr/java/packages/lib/ext" };
     
+    
+    /** Set to true if the VM is gcj, false otherwise */
+    public static final boolean IS_GNU_JAVA = checkVM();
+
     /** 
      * The TCP socket name. Default is 9267.
      * @see System property <code>php.java.bridge.tcp_socketname</code>
@@ -210,7 +225,13 @@ public final class Util {
      * @see System property <code>php.java.bridge.default_log_file</code>
      */
     public static String DEFAULT_LOG_FILE;
-	
+
+    /** The base directory of the PHP/Java Bridge. Usually /usr/php/modules/ or $HOME  */
+    public static String JAVABRIDGE_BASE;
+    
+    /** The library directory of the PHP/Java Bridge. Usually /usr/php/modules/lib or $HOME/lib */
+    public static String JAVABRIDGE_LIB;
+    
     private static String getProperty(Properties p, String key, String defaultValue) {
 	String s = null;
 	if(p!=null) s = p.getProperty(key);
@@ -226,9 +247,27 @@ public final class Util {
     public static String osName;
     /** Only for internal use */
     public static boolean IS_MONO;
+    /** Only for internal use */
+    public static String PHP_EXEC;
+    /** Only for internal use */
+    public static boolean EXT_JAVA_COMPATIBILITY;
+    /** Only for internal use */
+    public static File HOME_DIR;
 
     private static void initGlobals() {
-    	try {
+	try {
+	    HOME_DIR = new File(System.getProperty("user.home"));
+	} catch (Exception e) {
+	    HOME_DIR = null;
+	}
+	try {
+	    JAVABRIDGE_BASE = System.getProperty("php.java.bridge.base",  System.getProperty("user.home"));
+	    JAVABRIDGE_LIB =  JAVABRIDGE_BASE + File.separator +"lib";
+	} catch (Exception e) {
+	    JAVABRIDGE_BASE=".";
+	    JAVABRIDGE_LIB=".";	    
+	}
+	try {
     	    VM_NAME = "unknown";
 	    VM_NAME = System.getProperty("java.version")+"@" + System.getProperty("java.vendor.url");	    
 	} catch (Exception e) {/*ignore*/}
@@ -243,7 +282,8 @@ public final class Util {
     	    Util.loadMethod = Util.CLRAssembly.getMethod("Load", new Class[] {String.class});
     	    IS_MONO=true;
     	} catch (Exception e) {/*ignore*/}
-        Properties p = new Properties();
+
+    	Properties p = new Properties();
 	try {
 	    InputStream in = Util.class.getResourceAsStream("global.properties");
 	    p.load(in);
@@ -256,14 +296,22 @@ public final class Util {
 	} catch (Throwable t) {
 	    //t.printStackTrace();
 	};
-	TCP_SOCKETNAME = getProperty(p, "TCP_SOCKETNAME", "9267");
+	if (IS_MONO) {
+	    	EXTENSION_TCP_PORT_BASE = 9167;
+		TCP_SOCKETNAME = String.valueOf(Integer.parseInt(getProperty(p, "TCP_SOCKETNAME", "9267"))-100);
+	} else {
+	    	EXTENSION_TCP_PORT_BASE = 9267;
+		TCP_SOCKETNAME = getProperty(p, "TCP_SOCKETNAME", String.valueOf(EXTENSION_TCP_PORT_BASE));
+	}
 	EXTENSION_NAME = getProperty(p, "EXTENSION_DISPLAY_NAME", "JavaBridge");
+	PHP_EXEC = getProperty(p, "PHP_EXEC", null);
 	try {
 	    String s = getProperty(p, "DEFAULT_LOG_LEVEL", "3");
 	    DEFAULT_LOG_LEVEL = Integer.parseInt(s);
 	    Util.logLevel=Util.DEFAULT_LOG_LEVEL; /* java.log_level in php.ini overrides */
 	} catch (NumberFormatException e) {/*ignore*/}
 	DEFAULT_LOG_FILE = getProperty(p, "DEFAULT_LOG_FILE", Util.EXTENSION_NAME+".log");
+	EXT_JAVA_COMPATIBILITY = "true".equals(getProperty(p, "EXT_JAVA_COMPATIBILITY", "false"));
 	String separator = "/-+.,;: ";
 	try {
 	    String val = System.getProperty("os.arch").toLowerCase();
@@ -309,7 +357,7 @@ public final class Util {
     }
     
     /**
-     * Display a warning if logLevel >= 1
+     * Display a warning if logLevel &gt;= 1
      * @param msg The warn message
      */
     public static void warn(String msg) {
@@ -656,7 +704,18 @@ public final class Util {
 	} catch (UnknownHostException e) {/*ignore*/}
 	return addr;
     }
-	
+    /**
+     * Returns the canonical windows file. For example c:\program files instead of c:\programme
+     * @param path The path, may be an empty string.
+     * @return the canonical file.
+     */
+    public static File getCanonicalWindowsFile (String path) {
+	    try {
+	        return new File(path).getCanonicalFile();
+        } catch (IOException e) {
+	        return new File(path);
+        }
+    }
     /**
      * Checks if the cgi binary buf-&lt;os.arch&gt;-&lt;os.name&gt;.sh or buf-&lt;os.arch&gt;-&lt;os.name&gt;.exe or buf-&lt;os.arch&gt;-&lt;os.name&gt; exists.
      * @param buf The base name, e.g.: /opt/tomcat/webapps/JavaBridge/WEB-INF/cgi/php-cgi
@@ -745,7 +804,8 @@ public final class Util {
 	private File homeDir;
 	private Map env;
 	private boolean tryOtherLocations;
-
+	private boolean preferSystemPhp;
+	
 	protected String[] getArgumentArray(String[] php, String[] args) {
 	    LinkedList buf = new LinkedList();
 	    buf.addAll(java.util.Arrays.asList(php));
@@ -754,7 +814,6 @@ public final class Util {
 	    }
 	    return  (String[]) buf.toArray(new String[buf.size()]);
 	}
-	private static final String PHP_EXEC = System.getProperty("php.java.bridge.php_exec");
 	protected void start() throws NullPointerException, IOException {
 	    File location;
 	    Runtime rt = Runtime.getRuntime();
@@ -766,36 +825,59 @@ public final class Util {
 	    if(args==null) args=new String[]{null};
 	    String phpExec = args[0];
 	    String[] cgiBinary = null;
-            if(phpExec != null && (cgiBinary=checkCgiBinary(new StringBuffer(phpExec))) != null) php = cgiBinary;
-            
-            /*
-             * ... resolve it ..
-             */            
-            if(PHP_EXEC==null && tryOtherLocations && php[0]==null) {
-		for(int i=0; i<DEFAULT_CGI_LOCATIONS.length; i++) {
-		    location = new File(DEFAULT_CGI_LOCATIONS[i]);
-		    if(location.exists()) {php[0] = location.getAbsolutePath(); break;}
+	    if(PHP_EXEC==null) {
+	      if(!preferSystemPhp) {
+		if(phpExec != null && 
+				((cgiBinary=checkCgiBinary(new StringBuffer(phpExec))) != null)) php = cgiBinary;
+		/*
+		 * ... resolve it ..
+		 */            
+		if(tryOtherLocations && php[0]==null) {
+		    for(int i=0; i<DEFAULT_CGI_LOCATIONS.length; i++) {
+			location = new File(DEFAULT_CGI_LOCATIONS[i]);
+			if(location.exists()) {php[0] = location.getAbsolutePath(); break;}
+		    }
 		}
-            }
+	      } else {
+		/*
+		 * ... resolve it ..
+		 */            
+		if(tryOtherLocations && php[0]==null) {
+		    for(int i=0; i<DEFAULT_CGI_LOCATIONS.length; i++) {
+			location = new File(DEFAULT_CGI_LOCATIONS[i]);
+			if(location.exists()) {
+			    php[0] = location.getAbsolutePath(); 
+			    break;
+			}
+		    }
+		}
+		if(phpExec != null && 
+				(php[0]==null &&  (cgiBinary=checkCgiBinary(new StringBuffer(phpExec))) != null)) php = cgiBinary;
+	      }
+	    }
             if(php[0]==null && tryOtherLocations) php[0]=PHP_EXEC;
-            if(php[0]==null && args[0]!=null && (new File(args[0]).exists())) php[0]=args[0];
+            if(php[0]==null && phpExec!=null && (new File(phpExec).exists())) php[0]=phpExec;
             if(php[0]==null) php[0]="php-cgi";
             if(Util.logLevel>3) Util.logDebug("Using php binary: " + java.util.Arrays.asList(php));
 
             /*
              * ... and construct a new argument array for this specific process.
              */
+            if(cgiBinary ==null)
+        	homeDir = HOME_DIR; // do not read ./php.ini
+            
             if(homeDir!=null &&!homeDir.exists()) homeDir = null;
 	    String[] s = getArgumentArray(php, args);
 	    
 	    proc = rt.exec(s, hashToStringArray(env), homeDir);
 	    if(Util.logLevel>3) Util.logDebug("Started "+ java.util.Arrays.asList(s));
         }
-	protected Process(String[] args, File homeDir, Map env, boolean tryOtherLocations) {
+	protected Process(String[] args, File homeDir, Map env, boolean tryOtherLocations, boolean preferSystemPhp) {
 	    this.args = args;
 	    this.homeDir = homeDir;
 	    this.env = env;
 	    this.tryOtherLocations = tryOtherLocations;
+	    this.preferSystemPhp = preferSystemPhp;
 	}
         /**
 	 * Starts a CGI process and returns the process handle.
@@ -808,12 +890,25 @@ public final class Util {
 	 * @throws IOException
 	 * @see Util#checkCgiBinary(StringBuffer)
 	 */	  
-        public static Process start(String[] args, File homeDir, Map env, boolean tryOtherLocations) throws IOException {
-            Process proc = new Process(args, homeDir, env, tryOtherLocations);
+        public static Process start(String[] args, File homeDir, Map env, boolean tryOtherLocations, boolean preferSystemPhp) throws IOException {
+            Process proc = new Process(args, homeDir, env, tryOtherLocations, preferSystemPhp);
             proc.start();
             return proc;
         }
 
+        public static class PhpException extends Exception {
+	    private static final long serialVersionUID = 767047598257671018L;
+	    private String errorString;
+	    public PhpException(String errorString) {
+		super(errorString);
+		this.errorString = errorString;
+	    }
+	    public String getError() {
+		return errorString;
+	    }
+	};
+
+	public void checkError() throws PhpException {}
 
         /* (non-Javadoc)
          * @see java.lang.Process#getOutputStream()
@@ -865,21 +960,11 @@ public final class Util {
     public static class ProcessWithErrorHandler extends Process {
 	StringBuffer error = null;
 	InputStream in = null;
+	OutputStream err = null;
 
-	public static class PhpException extends RuntimeException {
-	    private static final long serialVersionUID = 767047598257671018L;
-	    private String errorString;
-	    public PhpException(String errorString) {
-		super(errorString);
-		this.errorString = errorString;
-	    }
-	    public String getError() {
-		return errorString;
-	    }
-	};
-
-	protected ProcessWithErrorHandler(String[] args, File homeDir, Map env, boolean tryOtherLocations) throws IOException {
-	    super(args, homeDir, env, tryOtherLocations);
+	protected ProcessWithErrorHandler(String[] args, File homeDir, Map env, boolean tryOtherLocations, boolean preferSystemPhp, OutputStream err) throws IOException {
+	    super(args, homeDir, env, tryOtherLocations, preferSystemPhp);
+	    this.err = err;
 	}
 	protected void start() throws IOException {
 	    super.start();
@@ -890,13 +975,12 @@ public final class Util {
          * @param s The error string
          * @return The fatal error or null
          */	
-	protected String checkError(String s) {
-	    return Util.checkError(s);
+	public void checkError() throws PhpException {
+	    String errorString = error==null?null:Util.checkError(error.toString());
+	    if(errorString!=null) throw new PhpException(errorString);
 	}
 	public void destroy() {
-	    try {proc.destroy();} catch (Exception e) {}
-	    String errorString = error==null?null:checkError(error.toString());
-	    if(errorString!=null) throw new PhpException(errorString);
+	    proc.destroy();
 	}
 	private synchronized void readErrorStream() {
 	    byte[] buf = new byte[BUF_SIZE];
@@ -904,10 +988,11 @@ public final class Util {
 	    try { 
 		in =  proc.getErrorStream();
 		while((c=in.read(buf))!=-1) {
-		    String s = new String(buf, 0, c, ASCII); 
-		    Util.logError(s);
-		    if(error==null) error = new StringBuffer(s);
-		    else error.append(s);
+			err.write(buf, 0, c);
+			String s = new String(buf, 0, c, ASCII); 
+			if(Util.logLevel>4) Util.logError(s);
+			if(error==null) error = new StringBuffer(s);
+			else error.append(s);
 		}
 	    } catch (IOException e) {
 		e.printStackTrace();
@@ -934,8 +1019,8 @@ public final class Util {
          * @throws IOException
          * @see Util#checkCgiBinary(StringBuffer)
          */
-        public static Process start(String[] args, File homeDir, Map env, boolean tryOtherLocations) throws IOException {
-            Process proc = new ProcessWithErrorHandler(args, homeDir, env, tryOtherLocations);
+        public static Process start(String[] args, File homeDir, Map env, boolean tryOtherLocations, boolean preferSystemPhp, OutputStream err) throws IOException {
+            Process proc = new ProcessWithErrorHandler(args, homeDir, env, tryOtherLocations, preferSystemPhp, err);
             proc.start();
             return proc;
         }
@@ -943,9 +1028,9 @@ public final class Util {
 
     /**
      * @return The thread context class loader.
-     */
+      */
     public static ClassLoader getContextClassLoader() {
-        return JavaBridgeClassLoader.DEFAULT_CLASS_LOADER;
+          return JavaBridgeClassLoader.getDefaultClassLoader(JavaBridgeClassLoader.getContextClassLoader());
     }
     /** Redirect System.out and System.err to the configured logFile or System.err.
      * System.out is always redirected, either to the logFile or to System.err.
@@ -1109,8 +1194,8 @@ public final class Util {
         return s;
     }
 
-    public static ThreadPool createThreadPool(String name) {
-        ThreadPool pool = null;
+    public static AppThreadPool createThreadPool(String name) {
+        AppThreadPool pool = null;
         int maxSize = 20;
         try {
         	maxSize = Integer.parseInt(Util.THREAD_POOL_MAX_SIZE);
@@ -1118,8 +1203,119 @@ public final class Util {
         	Util.printStackTrace(t);
         }
         if(maxSize>0) {
-            pool = new ThreadPool(name, maxSize);
+            pool = new AppThreadPool(name, maxSize);
 	}
         return pool;
     }
+    
+    
+    /**
+     * parse java.log_file=@HOST:PORT
+     * @param logFile The log file from the PHP .ini file
+     * @return true, if we can use the log4j logger, false otherwise.
+     */
+    public static boolean setConfiguredLogger(String logFile) {
+        try {
+	  return tryConfiguredChainsawLogger(logFile);
+	} catch (Exception e) {
+	  printStackTrace(e);
+	  Util.setLogger(new FileLogger());
+	}
+	return true;
+    }
+    private static final class ConfiguredChainsawLogger extends ChainsawLogger {
+        private String host;
+	private int port;
+	private ConfiguredChainsawLogger(String host, int port) {
+	    super();
+	    this.host=host;
+	    this.port=port;
+        }
+        public static ConfiguredChainsawLogger createLogger(String host, int port) throws Exception {
+            ConfiguredChainsawLogger logger = new ConfiguredChainsawLogger(host, port);
+            logger.init();
+	    return logger;
+        }
+        public void configure(String host, int port) throws Exception {
+            host = this.host!=null ? this.host : host;
+            port = this.port > 0 ? this.port : port;
+            super.configure(host, port);
+        }
+    }
+    /**
+     * parse java.log_file=@HOST:PORT
+     * @param logFile The log file from the PHP .ini file
+     * @return true, if we can use the log4j logger, false otherwise.
+     * @throws Exception
+     */
+    private static boolean tryConfiguredChainsawLogger(String logFile) throws Exception {
+	if(logFile!=null && logFile.length()>0 && logFile.charAt(0)=='@') {
+	    logFile=logFile.substring(1, logFile.length());
+	    int idx = logFile.indexOf(':');
+	    int port = -1;
+	    String host = null;
+	    if(idx!=-1) {
+		String p = logFile.substring(idx+1, logFile.length());
+		if(p.length()>0) port = Integer.parseInt(p);
+		host = logFile.substring(0, idx);
+	    } else {
+		if(logFile.length()>0) host = logFile;
+	    }
+	    Util.setLogger(ConfiguredChainsawLogger.createLogger(host, port));
+	    return true;
+	}
+	return false;
+    }
+
+    public static String formatDateTime(long ms) {
+	java.sql.Timestamp t = new java.sql.Timestamp(ms);
+	DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.LONG, Locale.ENGLISH);
+	formatter.setTimeZone(TimeZone.getTimeZone("GMT"));
+	String str =  formatter.format(t);
+	return str;
+    }
+    public static void destroy () {
+	    Util.logLevel = 0;
+    }
+    public static int getMBeanProperty(String pattern, String property) {
+            try {
+             Class objectNameClazz = Class.forName("javax.management.ObjectName");
+             Constructor constructor = objectNameClazz.getConstructor(new Class[]{String.class});
+             Object objectName = constructor.newInstance(new Object[]{pattern});
+             
+             
+             Class clazz = Class.forName("javax.management.MBeanServerFactory");
+             Method method = clazz.getMethod("findMBeanServer", new Class[]{String.class});
+             ArrayList servers = (ArrayList)method.invoke(clazz, new Object[]{null});
+             Object server = servers.get(0);
+             
+             Class mBeanServerClazz = Class.forName("javax.management.MBeanServer");
+             clazz = Class.forName("javax.management.QueryExp");
+             method = mBeanServerClazz.getMethod("queryMBeans", new Class[]{objectNameClazz, clazz});
+             
+             Set s = (Set)method.invoke(server, new Object[]{objectName, null});
+             Iterator ii = s.iterator(); 
+             
+             if (ii.hasNext()) {
+        	     clazz = Class.forName("javax.management.ObjectInstance");
+             method = clazz.getMethod("getObjectName", Util.ZERO_PARAM);
+             objectName = method.invoke(ii.next(), Util.ZERO_ARG);
+             
+             method = mBeanServerClazz.getMethod("getAttribute", new Class[]{objectNameClazz, String.class});
+        	     Object result = method.invoke(server, new Object[]{objectName, property});
+        	     return Integer.parseInt(String.valueOf(result));
+             }
+	} catch (Exception t) {
+		if (Util.logLevel>5) Util.printStackTrace(t);
+	}
+	return 0;
+   }
+
+    static final boolean checkVM() {
+	try {
+	    return "libgcj".equals(System.getProperty("gnu.classpath.vm.shortname"));
+	} catch (Throwable t) {
+	    return false;
+	}
+    }
 }
