@@ -33,6 +33,7 @@ import php.java.bridge.ISession;
 import php.java.bridge.JavaBridge;
 import php.java.bridge.JavaBridgeClassLoader;
 import php.java.bridge.SessionFactory;
+import php.java.bridge.SimpleJavaBridgeClassLoader;
 import php.java.bridge.Util;
 
 
@@ -103,13 +104,11 @@ public final class ContextFactory extends SessionFactory implements IContextFact
 
     private boolean invalid=false;
 
-    private JavaBridge bridge = null;
-
     private String id;
     private long timestamp;
 
     private ICredentials contextServer = null;
-    private IContextFactory visitor; 
+    private IContextFactoryVisitor visitor; 
 
     private static long counter = 0;
     private static synchronized String addNext(String webContext, ContextFactory thiz) {
@@ -120,8 +119,8 @@ public final class ContextFactory extends SessionFactory implements IContextFact
         return id;
     }
     private static synchronized void remove(String id) {
-	Object ob = liveContexts.remove(id);
-        if(Util.logLevel>4) Util.logDebug("removed context: " + ob + ", # of contexts: " + contexts.size());
+	ContextFactory ob = (ContextFactory) liveContexts.remove(id);
+        if(Util.logLevel>4) Util.logDebug("contextfactory: removed context: " + ob.visitor + ", # of contexts: " + contexts.size());
     }
     private static synchronized ContextFactory moveContext(String id) {
         Object o;
@@ -133,21 +132,9 @@ public final class ContextFactory extends SessionFactory implements IContextFact
       super();
       timestamp = System.currentTimeMillis();
       id=addNext(webContext, this);
-      if(Util.logLevel>4) Util.logDebug("new context: " + id + ", # of contexts: " + contexts.size());
+      if(Util.logLevel>4) Util.logDebug("contextfactory: new context: " + id + " for web context" + webContext + ", # of contexts: " + contexts.size());
     }
 
-    /**
-     * Create a new ContextFactory and add it to the list of context factories kept by this classloader.
-     * @param webContext The current web context, use ContextFactory.EMPTY_CONTEXT 
-     * if servlet web contexts are not available, config.getServletContext().getRealPath("") otherwise.
-     * @return The created ContextFactory.
-     * @see php.java.bridge.http.ContextFactory#get(String, ICredentials)
-     */
-    public static IContextFactory addNew(String webContext) {
-    	ContextFactory factory = new ContextFactory(webContext);
-    	factory.visitor = factory;
-    	return factory;
-    }
     /**
      * Create a new simple ContextFactory (a factory which creates an emulated JSR223 context) and add it 
      * to the list of context factories kept by this classloader.
@@ -169,13 +156,13 @@ public final class ContextFactory extends SessionFactory implements IContextFact
      * @param id The ID
      * @param server Your context server.
      * @return The ContextFactory or null.
-     * @see #addNew(String)
+     * @see php.java.bridge.http.SimpleContextFactory#addNew(String)
      * @throws SecurityException if id belongs to a different ContextServer.
      */
     /* See PhpJavaServlet#contextServer, http.ContextRunner#contextServer and 
      * JavaBridgeRunner#ctxServer. */
     public static IContextFactory get(String id, ICredentials server) {
-        if(server == null) throw new NullPointerException("server");
+	if(server == null) return peek(id);
 
         ContextFactory factory = moveContext(id); 
         if(factory==null) return null;
@@ -185,46 +172,84 @@ public final class ContextFactory extends SessionFactory implements IContextFact
             throw new SecurityException("Illegal access");
         return factory.visitor;
     }
-    /* (non-Javadoc)
-     * @see php.java.bridge.http.IContextFactory#recycle(php.java.bridge.http.ContextFactory)
+    /**
+     * Only for internal use.
+     * The same as {@link #get(String, php.java.bridge.http.ContextFactory.ICredentials)} with the second argument set to null.
+     * 
+     * The servlet may use this procedure to check for a new context factory.
+     * @param id The existing id.
+     * @return The factory or null
+     * @throws IllegalStateException if the context is in use.
      */
-    public void recycle(ContextFactory target) {}
+    public static synchronized IContextFactory peek(String id) {
+	if(liveContexts.get(id)!=null) throw new IllegalStateException(id);
+	ContextFactory factory = (ContextFactory) contexts.get(id);
+	return factory == null ? null : factory.visitor;
+    }
+    
+    /*
+     * Attach the new factory to the persistent one
+     * @factory The fresh factory from the servlet
+     */ 
+    private void switchContext(ContextFactory factory) {	
+	JavaBridge bridge = getBridge();
+	JavaBridge newBridge = factory.checkBridge();
+	if(newBridge!=null) { // set the new bridge which keeps a reference to the fresh context
+	    if(Util.logLevel>4) Util.logDebug("contextfactory: setting new bridge. visited: " + bridge.getFactory() + " <= visitor: " + newBridge.getFactory()); 
+	    
+	    /* set the new bridge and the associated threadContextClassLoader, will be reset in recycle() */
+	    bridge.request.setBridge(newBridge);
+	    bridge = newBridge;
+	    
+	    /* remove the fresh context factory and/or move it to the liveContexts and store a reference to it */
+	    factory.visitor.removeOrphaned();
+	    accept(factory.visitor);
+	} else {
+		throw new IllegalStateException("recycle empty context");
+	}
+	
+	if(factory.getClassLoader() !=bridge.getClassLoader().getDefaultClassLoader()) 
+	    throw new IllegalStateException("class loader");
+	if(Util.logLevel>4) Util.logDebug("contextfactory: " +this + " is swiching thread context, using classloader: " + System.identityHashCode( factory.getClassLoader().getParent()));
+
+	invalid=false;
+	if(Util.logLevel>4) Util.logDebug("contextfactory: accepted visitor: " + factory.visitor);
+    }
     /* (non-Javadoc)
      * @see php.java.bridge.http.IContextFactory#recycle(java.lang.String)
      */
     public void recycle(String id) throws SecurityException {
-        IContextFactory target = ((ContextFactory)contexts.get(id)).visitor;
-        target.removeOrphaned();
-        target.recycle(this);
-    }
+	ContextFactory factory = null;
+	factory=((ContextFactory)contexts.get(id));
+	if(factory == null || factory == this) return;
+
+	switchContext(factory);
+   }
     /**
      * Recycle the factory for new reqests.
      */    
-    public synchronized void recycle() {
+    public void recycle() {
+	if(Util.logLevel>=4) Util.logDebug("contextfactory: finish context called (recycle context factory) " + this.visitor);
 	super.recycle();
-	contextServer=null;
+	visitor.recycle();
 	invalid=true;
-	notify();
+	visitor.invalidate();
+	
+	if(bridge!=null) 
+	    bridge.getClassLoader().switcheThreadContextClassLoader();
     }
     
     /* (non-Javadoc)
      * @see php.java.bridge.http.IContextFactory#destroy()
      */
-    public synchronized void destroy() {
-	if(Util.logLevel>4) Util.logDebug("context finished: " +getId());
+    public void destroy() {
+	if(Util.logLevel>4) Util.logDebug("contextfactory: context destroyed (remove context factory): " +visitor);
 	remove(getId());
 	bridge=null;
 	invalid=true;
-	notify();
+	visitor.invalidate();
    }
     
-    /**
-     * @deprecated
-     * @see #destroyAll()
-     */
-    public static void removeAll() {
-        destroyAll();
-    }
     private boolean isInitialized() {
         return (contextServer!=null);
     }
@@ -246,11 +271,9 @@ public final class ContextFactory extends SessionFactory implements IContextFact
         for(Iterator ii=contexts.values().iterator(); ii.hasNext();) {
 	    ContextFactory ctx = ((ContextFactory)ii.next());
 	    if(ctx.timestamp+ORPHANED_TIMEOUT<timestamp) {
-	        synchronized(ctx) {
-	            ctx.invalid=true;
-	            ctx.notify();
-	        }
-	        if(Util.logLevel>4) Util.logDebug("Orphaned context: " + ctx.getId() + " removed.");
+		ctx.invalid=true;
+	        ctx.visitor.invalidate();
+	        if(Util.logLevel>4) Util.logDebug("contextfactory: Orphaned context: " + ctx.visitor + " removed.");
 	        ii.remove();
 	    }
 	}        
@@ -261,28 +284,31 @@ public final class ContextFactory extends SessionFactory implements IContextFact
      * @see php.java.bridge.http.ContextServer
      */
     public static synchronized void destroyAll() {
-	destroyOrphaned();
+        for(Iterator ii=contexts.values().iterator(); ii.hasNext();) {
+	    ContextFactory ctx = ((ContextFactory)ii.next());
+	    ctx.invalid=true;
+	    ctx.visitor.invalidate();
+	    if(Util.logLevel>4) Util.logDebug("contextfactory: Orphaned context: " + ctx.visitor + " removed.");
+	    ii.remove();
+	}
 	for(Iterator ii=contexts.values().iterator(); ii.hasNext();) {
 	    ContextFactory ctx = ((ContextFactory)ii.next());
-	    synchronized(ctx) {
-		ctx.invalid=true;
-		ctx.notify();
-	    }
+	    ctx.invalid=true;
+	    ctx.visitor.invalidate();
 	    ii.remove();
 	}
     }
-    
     /* (non-Javadoc)
      * @see php.java.bridge.http.IContextFactory#waitFor()
      */
-    public synchronized void waitFor() throws InterruptedException {
-    	if(!invalid) wait();
+    public void waitFor() throws InterruptedException {
+	    visitor.waitFor();
     }
     /* (non-Javadoc)
      * @see php.java.bridge.http.IContextFactory#waitFor()
      */
-    public synchronized void waitFor(long timeout) throws InterruptedException {
-	if(!invalid) wait(timeout);
+    public void waitFor(long timeout) throws InterruptedException {
+	    visitor.waitFor(timeout);
     }
     /* (non-Javadoc)
      * @see php.java.bridge.http.IContextFactory#getId()
@@ -294,7 +320,7 @@ public final class ContextFactory extends SessionFactory implements IContextFact
      * @see php.java.bridge.http.IContextFactory#toString()
      */
     public String toString() {
-	return "Context# " +id + ", isInitialized: " + isInitialized();
+	return "Context# " +id + ", isInitialized: " + isInitialized() + ", isValid: " + !invalid;
     }
     /**
      * Returns the context.
@@ -308,28 +334,12 @@ public final class ContextFactory extends SessionFactory implements IContextFact
      * Set the Context into this factory.
      * Should be called by Context.addNew() only.
      * @param context The context.
-     * @see #addNew(String)
+     * @see php.java.bridge.http.SimpleContextFactory#addNew(String)
      */
     public void setContext(IContext context) {
         visitor.setContext(context);
     }
-    /* (non-Javadoc)
-     * @see php.java.bridge.http.IContextFactory#setBridge(php.java.bridge.JavaBridge)
-     */
-    public void setBridge(JavaBridge bridge) {
-	this.bridge = bridge;
-    }
-    /* (non-Javadoc)
-     * @see php.java.bridge.http.IContextFactory#getBridge()
-     */
-    public JavaBridge getBridge() {
-        if(bridge != null) return bridge;
-        setBridge(new JavaBridge());
-        bridge.setClassLoader(new JavaBridgeClassLoader());
-        bridge.setSessionFactory(this);
-	return bridge;
-    }
-    private void setVisitor(IContextFactory newVisitor) {
+    private void setVisitor(IContextFactoryVisitor newVisitor) {
         visitor = newVisitor;
     }
     /**
@@ -337,7 +347,7 @@ public final class ContextFactory extends SessionFactory implements IContextFact
      * @param visitor The custom ContextFactory
      */
     public void accept(IContextFactoryVisitor visitor) {
-        visitor.visit(this);
+	visitor.visit(this);
         setVisitor(visitor);
     }
     
@@ -360,9 +370,17 @@ public final class ContextFactory extends SessionFactory implements IContextFact
 	return visitor.getSession(name, clientIsNew, timeout);
     }
     public synchronized void removeOrphaned() {
-	Object ob = contexts.remove(id);
-        if(Util.logLevel>4) Util.logDebug("removed empty context: " + ob + ", # of contexts: " + contexts.size());
+	ContextFactory ob = (ContextFactory) contexts.remove(id);
+        if(Util.logLevel>4) Util.logDebug("contextfactory: removed empty context: " + (ob!=null?String.valueOf(ob.visitor):"<already handled>") + ", # of contexts: " + contexts.size()+", # of live contexts: "+ liveContexts.size());
 	invalid=true;
-	notify();
+    }
+    public void setClassLoader(ClassLoader loader) {
+	visitor.setClassLoader(loader);
+    }
+    public SimpleJavaBridgeClassLoader getJavaBridgeClassLoader() {
+	return new JavaBridgeClassLoader(getClassLoader());
+    }
+    public ClassLoader getClassLoader() {
+	return visitor.getClassLoader();
     }
 }
