@@ -24,8 +24,8 @@ package php.java.bridge.http;
  * OTHER DEALINGS IN THE SOFTWARE.
  */
 
+import php.java.bridge.IJavaBridgeFactory;
 import php.java.bridge.ISession;
-import php.java.bridge.JavaBridge;
 
 /**
  * Interface that the ContextFactories must implement.
@@ -33,39 +33,18 @@ import php.java.bridge.JavaBridge;
  * @author jostb
  *
  */
-public interface IContextFactory {
+public interface IContextFactory extends IJavaBridgeFactory {
 
   /**
-   * Synchronize the current state with id.
    * <p>
-   * When persistent connections are used, the bridge instances recycle their context factories (persistent
-   * php clients store their context id, so that they don't have to aquire a new one).
-   * However, a client of a php client may have passed a fresh context id. 
-   * If this happened, the bridge calls this method, 
-   * which may update the current context with the fresh values from id.</p>
-   * <p>Typically the ContextFactory implements this method. It should find the ContextFactory for id,
-   * check that the Factory is not in use (and throw a SecurityException, if isInitialized() returns true),
-   * remove it by calling removeOrphaned() and 
-   * call its recycle() method passing it the current ContextFactory. 
+   * Update the context factory with the new JavaBridge obtained from the servlet
    * </p>
    * @param id The fresh id
-   * @throws NullPointerException if the current id is not initialized
-   * @throws SecurityException if the found ContextFactory is initialized.
-   * @see #recycle(ContextFactory)
-   * @see php.java.bridge.JavaBridge#recycle()
+   * @see php.java.bridge.http.ContextFactory#recycle()
+   * @see php.java.bridge.Request#setBridge(php.java.bridge.JavaBridge)
+   * @see php.java.bridge.Request#recycle()
    */
-  public void recycle(String id) throws SecurityException;
-  
-  /**
-   * Typically the visitor implements this method, it should attach itself to the target by calling
-   * <code>target.accept(this)</code>.
-   * @see #recycle(String)
-   * @see php.java.bridge.http.ContextFactory#accept(IContextFactoryVisitor)
-   * @see php.java.bridge.http.IContextFactoryVisitor#visit(ContextFactory)
-   * @see php.java.bridge.http.SimpleContextFactory
-   * @param target The persistent ContextFactory.
-   */
-  public void recycle(ContextFactory target);
+    public void recycle(String id) throws SecurityException;
 
   /**
    * Removes the context factory from the classloader's list of context factories
@@ -114,12 +93,6 @@ public interface IContextFactory {
    * @see php.java.bridge.http.ContextFactory#addNew(String)
    */
   public void setContext(IContext context);
-
-  /**
-   * Return the JavaBridge.
-   * @return Returns the bridge.
-   */
-  public JavaBridge getBridge();
   
   /**
    * @param name The session name. If name is null, the name PHPSESSION will be used.
@@ -130,4 +103,15 @@ public interface IContextFactory {
    */
    public ISession getSession(String name, boolean clientIsNew, int timeout);
 
+   /**
+    * Set the class loader obtained from the current servlet into the context.
+    * @param loader The currentThreadContextClassLoader
+    */
+   public void setClassLoader(ClassLoader loader);
+   
+   /**
+    * Get the class loader from the servlet.
+    * @return The currentThreadContextClassLoader of the servlet.
+    */
+   public ClassLoader getClassLoader();
 }
