@@ -36,4 +36,9 @@ public interface IContextFactoryVisitor extends IContextFactory {
    * @param visited The context factory
    */
   public void visit(ContextFactory visited);
+  
+  /**
+   * Called at the end of the visitor's life cycle
+   */
+  public void invalidate();
 }
