@@ -26,6 +26,8 @@ package php.java.bridge.http;
 
 import php.java.bridge.ISession;
 import php.java.bridge.JavaBridge;
+import php.java.bridge.Util;
+import php.java.bridge.SimpleJavaBridgeClassLoader;
 
 
 /**
@@ -33,10 +35,14 @@ import php.java.bridge.JavaBridge;
  *  
  * @see php.java.servlet.ServletContextFactory
  * @see php.java.script.PhpScriptContextFactory
- * @see php.java.faces.PhpFacesScriptContextFactory
  */
 public class SimpleContextFactory implements IContextFactoryVisitor {
     
+    /**
+     * The session object
+     */
+    protected ISession session;
+    
     /**
      * The visited ContextFactory
      */
@@ -47,9 +53,12 @@ public class SimpleContextFactory implements IContextFactoryVisitor {
      */
     protected IContext context;
     
+    private boolean isValid = true;
+    
     protected SimpleContextFactory(String webContext) {
   	visited = new ContextFactory(webContext);
   	visited.accept(this);
+    	setClassLoader(Util.getContextClassLoader());
     }
     
     public void recycle(String id) throws SecurityException {
@@ -58,27 +67,34 @@ public class SimpleContextFactory implements IContextFactoryVisitor {
 
     public void destroy() {
         visited.destroy();
+        session = null;
     }
     
-    /**
-     * Wait for the context factory to finish. The default implementation does nothing.
-     * Call visited.waitFor() if you want to wait.
-     */
-    public void waitFor() throws InterruptedException {
-        //visited.waitFor();
+    public synchronized void invalidate() {
+	    isValid = false;
+	    notifyAll(); // the servlet and the http proxy, see SimplePhpScriptEngine.release
     }
     /**
-     * Wait for the context factory to finish. The default implementation does nothing.
-     * Call visited.waitFor() if you want to wait.
+     * Wait for the context factory to finish. 
      */
-    public void waitFor(long timeout) throws InterruptedException {
-	//visited.waitFor(timeout);
+    public synchronized void waitFor() throws InterruptedException {
+	if(Util.logLevel>4) Util.logDebug("contextfactory: servlet is waiting for ContextFactory " +System.identityHashCode(this));
+	if (isValid) wait();
+	if(Util.logLevel>4) Util.logDebug("contextfactory: servlet done waiting for ContextFactory " +System.identityHashCode(this));
+    }
+    /**
+     * Wait for the context factory to finish. 
+     */
+    public synchronized void waitFor(long timeout) throws InterruptedException {
+	if(Util.logLevel>4) Util.logDebug("contextfactory: servlet is waiting for ContextFactory " +System.identityHashCode(this) + " for " +timeout+" ms");
+	if (isValid) wait(timeout);
+	if(Util.logLevel>4) Util.logDebug("contextfactory: servlet done waiting for ContextFactory " +System.identityHashCode(this)+ " for " +timeout+" ms");
     }    
     public String getId() { 
         return visited.getId();
     }
     public String toString() {
-	return "Visited: " + visited + ", Current: ";
+	return "ContextFactory: " + visited + ", SimpleContextFactory: " +getClass() + ", current loader: " + loader;
     }
     /**
      * Create a new context. The default implementation
@@ -93,9 +109,6 @@ public class SimpleContextFactory implements IContextFactoryVisitor {
         return context;
     }
 	
-    public void setBridge(JavaBridge bridge) {
-        visited.setBridge(bridge);
-    }
     public JavaBridge getBridge() {
         return visited.getBridge();
     }
@@ -103,22 +116,48 @@ public class SimpleContextFactory implements IContextFactoryVisitor {
         this.visited=visited;
     }
     public ISession getSession(String name, boolean clientIsNew, int timeout) {
-        return visited.getSimpleSession(name, clientIsNew, timeout);
+	if(session != null) return session;
+	return session = visited.getSimpleSession(name, clientIsNew, timeout);
     }
     public void setContext(IContext context) {
         this.context = context;
         this.context.setAttribute(IContext.JAVA_BRIDGE, getBridge(), IContext.ENGINE_SCOPE);
     }
-    public void recycle(ContextFactory visited) {
-        visited.accept(this);
-    }
     public void removeOrphaned() {
         visited.removeOrphaned();
     }
+
     /**
-     * @deprecated Use {@link #destroy()} instead
+     * Called by recycle at the end of the script
      */
-    public void remove() {
-        visited.destroy();
+    public void recycle() {
+	session = null;
+    }
+
+    private ClassLoader loader;
+    /**
+     * Return the current class loader.
+     * @return the current DynamicJavaBridgeClassLoader
+     */
+    public ClassLoader getClassLoader() {
+	return loader;
+    }
+
+    /**
+     * Return the JavaBridgeClassLoader, which wraps the
+     * DynamicJavaBridgeClassLoader
+     */
+    public SimpleJavaBridgeClassLoader getJavaBridgeClassLoader() {
+	return visited.getJavaBridgeClassLoader();
+    }
+
+    /**
+     * Set the current class loader
+     * @param loader The DynamicJavaBridgeClassLoader
+     */
+    public void setClassLoader(ClassLoader loader) {
+	if(loader==null) 
+	    throw new NullPointerException("loader");
+	this.loader = loader;
     }
 }
