@@ -26,12 +26,8 @@ package php.java.script;
 
 import java.io.Writer;
 
-import javax.script.SimpleScriptContext;
-
 import php.java.bridge.JavaBridgeRunner;
-import php.java.bridge.PhpProcedureProxy;
 import php.java.bridge.Util;
-import php.java.bridge.http.HttpServer;
 import php.java.bridge.http.IContext;
 
 /**
@@ -44,7 +40,7 @@ import php.java.bridge.http.IContext;
  * @author jostb
  *
  */
-public class PhpScriptContext extends SimpleScriptContext implements IContext, IPhpScriptContext {
+public class PhpScriptContext extends AbstractPhpScriptContext implements IContext, IPhpScriptContext {
     static JavaBridgeRunner bridgeRunner = null;
 
     static {
@@ -54,18 +50,7 @@ public class PhpScriptContext extends SimpleScriptContext implements IContext, I
 	    Util.printStackTrace(e);
 	}
     }
-
-    /**
-     * Return the http server associated with this VM.
-     * @return The http server.
-     */
-    public static HttpServer getHttpServer() {
-      return bridgeRunner;
-    }
-
-    private HttpProxy kont;
-    private Writer writer;
-	
+ 	
     /**
      * Create a standalone PHP script context.
      *
@@ -75,40 +60,29 @@ public class PhpScriptContext extends SimpleScriptContext implements IContext, I
     }
 
      private Writer getWriter(boolean isStandalone) {
-	 return isStandalone ? new PhpScriptLogWriter() : new PhpScriptWriter(System.out);
+	 return isStandalone ? PhpScriptLogWriter.getWriter(Util.getLogger()) : new PhpScriptWriter(System.out);
      }
     public Writer getWriter() {
 	if(writer == null) return writer =  getWriter(bridgeRunner.isStandalone());
+	else if(! (writer instanceof PhpScriptWriter)) setWriter(writer);
 	return writer;
     }
-	
-    /**
-     * Set the php continuation
-     * @param kont - The continuation.
-     */
-    public void setContinuation(HttpProxy kont) {
-	this.kont = kont;
+    private Writer getErrorWriter(boolean isStandalone) {
+	 return isStandalone ? PhpScriptLogWriter.getWriter(Util.getLogger()) : new PhpScriptWriter(System.err);
+    }
+    public Writer getErrorWriter() {
+	if(errorWriter == null) return errorWriter =  getErrorWriter(bridgeRunner.isStandalone());
+	else if(! (errorWriter instanceof PhpScriptWriter)) setErrorWriter(errorWriter);
+	return errorWriter;	
     }
 
-
-    /* (non-Javadoc)
-     * @see php.java.bridge.Invocable#call(php.java.bridge.PhpProcedureProxy)
-     */
-    /**@inheritDoc*/
-    public boolean call(PhpProcedureProxy kont) throws InterruptedException {
-	this.kont.call(kont);
-	return true;
+    public String getSocketName() {
+	    return bridgeRunner.getSocket().getSocketName();
     }
-
-
-    /**@inheritDoc*/
-    public void setWriter(Writer writer) {
-        this.writer = writer;
-    }
-
-
-    /**@inheritDoc*/
-    public HttpProxy getContinuation() {
-        return kont;
+    public String getContextString() {
+	    StringBuffer buf = new StringBuffer("http://127.0.0.1:");
+	    buf.append(getSocketName());
+	    buf.append("/JavaBridge");
+	    return buf.toString();
     }
 }
