@@ -33,15 +33,7 @@ import javax.servlet.http.HttpServletResponse;
  * objects are set to null.
  */
 public class RemoteHttpSessionFacade extends HttpSessionFacade {
-    protected RemoteHttpSessionFacade (ServletContext ctx, HttpServletRequest req, HttpServletResponse res, int timeout) {
-	super(ctx, req, res, timeout);
+    protected RemoteHttpSessionFacade (AbstractServletContextFactory ctxFactory, ServletContext ctx, HttpServletRequest req, HttpServletResponse res, boolean clientIsNew, int timeout) {
+	super(ctxFactory, ctx, req, res, clientIsNew, timeout);
     }
-    
-    /**
-     * throws IllegalStateException.
-     * @throws IllegalStateException
-     */
-    public HttpServletResponse getHttpServletResponse() {
-        throw new IllegalStateException("PHP not running in a servlet environment");
-    }
- }
+}
